# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.


"""Minimal Flask application example.

SPHINX-START

First install Invenio-Formatter by running:

.. code-block:: console

    $ pip install -e .[all]
    $ cd examples

Next, start the development server:

.. code-block:: console

   $ export FLASK_APP=app.py FLASK_DEBUG=1
   $ flask run

and open the example application in your browser:

.. code-block:: console

    $ open http://127.0.0.1:5000/

SPHINX-END

"""

import base64
import datetime
from os.path import dirname, join

import jinja2
from flask import Flask, render_template

from invenio_formatter import InvenioFormatter

# Create Flask application
app = Flask(__name__)
app.config["ALLOWED_HTML_TAGS"] = [
    "a",
]
app.config["ALLOWED_HTML_ATTRS"] = {
    "a": ["href"],
}
InvenioFormatter(app)

# Set jinja loader to first grab templates from the app's folder.
app.jinja_loader = jinja2.ChoiceLoader(
    [jinja2.FileSystemLoader(join(dirname(__file__), "templates")), app.jinja_loader]
)


@app.route("/", methods=["GET"])
def index():
    """Example format date."""
    mydate = datetime.date.today()
    malicious_script = b"<script>alert('I will hack Invenio!')</script>"
    base64_script = base64.b64encode(malicious_script).decode("UTF-8")
    content = "<a href='data:text/html;base64,{}'></a>".format(base64_script)
    return render_template("index.html", mydate=mydate, content=content)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Configuration for Invenio-Formatter."""

FORMATTER_BADGES_ALLOWED_TITLES = ["DOI"]
"""List of allowed titles in badges."""

FORMATTER_BADGES_TITLE_MAPPING = {}
"""Mapping of titles."""

FORMATTER_BADGES_MAX_CACHE_AGE = 0
"""The maximum amount of time a badge will be considered fresh."""



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
# Copyright (C) 2025 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Jinja utilities for Invenio."""

from importlib.metadata import PackageNotFoundError, distribution

from . import config
from .filters.datetime import (
    format_arrow,
    from_isodate,
    from_isodatetime,
    naturaltime,
    to_arrow,
)
from .filters.html import sanitize_html
from .views import create_badge_blueprint


class InvenioFormatter(object):
    """Invenio-Formatter extension."""

    def __init__(self, app=None):
        """Extension initialization.

        :param app: The Flask application. (Default: ``None``)
        """
        if app:
            self.init_app(app)

    def init_app(self, app):
        """Flask application initialization.

        :param app: The Flask application.
        """
        self.init_config(app)

        # Install datetime helpers.
        app.jinja_env.filters.update(
            from_isodate=from_isodate,
            from_isodatetime=from_isodatetime,
            to_arrow=to_arrow,
            format_arrow=format_arrow,
            sanitize_html=sanitize_html,
            naturaltime=naturaltime,
        )

        if app.config["FORMATTER_BADGES_ENABLE"]:
            from invenio_formatter.context_processors.badges import badges_processor

            # Registration of context processors.
            app.context_processor(badges_processor)
            # Register blueprint.
            app.register_blueprint(
                create_badge_blueprint(app.config["FORMATTER_BADGES_ALLOWED_TITLES"])
            )

        app.extensions["invenio-formatter"] = self

    @staticmethod
    def init_config(app):
        """Initialize configuration.

        .. note:: If CairoSVG is installed then the configuration
            ``FORMATTER_BADGES_ENABLE`` is ``True``.

        :param app: The Flask application.
        """
        try:
            distribution("CairoSVG")
            has_cairo = True
        except PackageNotFoundError:
            has_cairo = False

        app.config.setdefault("FORMATTER_BADGES_ENABLE", has_cairo)

        for attr in dir(config):
            if attr.startswith("FORMATTER_"):
                app.config.setdefault(attr, getattr(config, attr))



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Badges context processor."""

import html
from base64 import b64encode

import cairosvg
from PIL import Image, ImageDraw, ImageFont


def get_text_length(*args):
    r"""Measure the size of string rendered with a TTF no-nomospaced fonts.

    :param \*args: List of strings to be measured.
    :returns: The length of the strings.
    """
    txt = Image.new("RGBA", (16, 16), (255, 255, 255, 0))
    d = ImageDraw.Draw(txt)
    font = ImageFont.truetype("DejaVuSans", 11)
    result = ()
    for value in args:
        result = result + (d.textlength(value, font=font),)
    return result


def generate_badge_svg(title, value, color="#007ec6"):
    """Generate the SVG.

    :param title: The badge title.
    :param value: The badge content.
    :param color: The badge color. (Default: ``'#007ec6'``)
    :returns: The SVG badge.
    """
    title = html.escape(title)
    value = html.escape(value)
    (title_length, value_length) = get_text_length(title, value)
    return """<svg xmlns="http://www.w3.org/2000/svg"
     width="{width}" height="20">
        <linearGradient id="b" x2="0" y2="100%">
            <stop offset="0" stop-color="#bbb" stop-opacity=".1"/>
            <stop offset="1" stop-opacity=".1"/>
        </linearGradient>
        <mask id="a" width="{width}" height="20">
            <rect width="{width}" height="20" rx="3"
            fill="#fff"/>
        </mask>
        <g mask="url(#a)">
            <path fill="#555" d="M0 0h{title_width}v20H0z" />
            <path fill="{color}"
            d="M{title_width} 0h{value_width}v20H{title_width}z"
            />
            <path fill="url(#b)" d="M0 0h{width}v20H0z" />
        </g>
        <g fill="#fff" text-anchor="middle" font-family="DejaVu Sans,
        Verdana,Geneva,sans-serif" font-size="11">
            <text x="{title_position}" y="15" fill="#010101"
            fill-opacity=".3">
                {title}
            </text>
            <text x="{title_position}" y="14">
                {title}
            </text>
            <text x="{value_position}"
            y="15" fill="#010101" fill-opacity=".3">
                {value}
            </text>
            <text x="{value_position}" y="14">
                {value}
            </text>
        </g>
    </svg>""".format(
        title_width=title_length + 11,
        value_width=value_length + 11,
        width=title_length + value_length + 22,
        title_position=title_length / 2 + 6,
        value_position=title_length + value_length / 2 + 16,
        title=title,
        value=value,
        color=color,
    )


def generate_badge_png(title, value, color="#007ec6"):
    """Generate the badge in PNG format."""
    badge = generate_badge_svg(title, value, color)
    return cairosvg.svg2png(badge)


def badges_processor():
    """Context processor for badges."""

    def badge_svg(title, value, color="#007ec6"):
        """Context processor function to generate SVG badges."""
        return generate_badge_svg(title, value, color)

    def badge_png(title, value, color="#007ec6"):
        """Context processor function to generate SVG badges."""
        png = generate_badge_png(title, value, color)
        png_base64 = b64encode(png)
        return "data:image/png;base64,{0}".format(png_base64)

    return dict(badge_svg=badge_svg, badge_png=badge_png)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Datetime Jinja filters."""

import arrow
from flask_babel import to_user_timezone


def from_isodate(value, strict=False):
    """Convert an ISO formatted date into a Date object.

    :param value: The ISO formatted date.
    :param strict: If value is ``None``, then if strict is ``True`` it returns
        the Date object of today, otherwise it returns ``None``.
        (Default: ``False``)
    :returns: The Date object or ``None``.
    """
    if value or strict:
        return arrow.get(value).date()


def from_isodatetime(value, strict=False):
    """Convert an ISO formatted datetime into a Date object.

    :param value: The ISO formatted datetime.
    :param strict: If value is ``None``, then if strict is ``True`` it returns
        the Date object of today, otherwise it returns ``None``.
        (Default: ``False``)
    :returns: The Date object or ``None``.
    """
    if value or strict:
        return arrow.get(value).datetime


def format_arrow(value, format_string):
    """Format an arrow datetime object.

    :param value: The arrow datetime object.
    :param format_string: The date format string
    :returns: Returns a string representation of the given arrow datetime
        object, formatted according to the given format string.

    .. note::
        Do not use this filter to format date/times presented to an end
        user. Instead use ``datetimeformat`` or ``dateformat`` from
        Invenio-I18N.
    """
    assert isinstance(value, arrow.Arrow)
    return value.format(format_string)


def to_arrow(value):
    """Convert a Date object to an arrow datetime object."""
    return arrow.get(value)


def naturaltime(value):
    """Get humanized version of time."""
    arrow_value = to_arrow(value)
    humanized = arrow_value.humanize()

    return humanized



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2019 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""HTML sanitisation Jinja filters."""

import bleach
from flask import current_app


def sanitize_html(value, tags=None, attributes=None):
    """Sanitize HTML.

    :param tags: Allowed HTML ``tags``. Configuration set by Invenio-Config.
    :param attributes: Allowed HTML ``attributes``. Configuration set by
        Invenio-Config.

    Use this function when you need to include unescaped HTML that contain
    user provided data.
    """
    return bleach.clean(
        value,
        tags=tags or current_app.config.get("ALLOWED_HTML_TAGS", []),
        attributes=attributes or current_app.config.get("ALLOWED_HTML_ATTRS", {}),
        strip=True,
    ).strip()



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""View method for Invenio-Formatter."""

import hashlib
from datetime import datetime as dt
from datetime import timedelta

from flask import Blueprint, Response, current_app, request


def valid_css_color(color):
    """Test for a valid CSS color (RGB, RRGGBB, or RRGGBBAA hex strings)."""
    if len(color) not in (3, 6, 8):
        return False
    valid_chars = set("0123456789abcdef")
    return all(char in valid_chars for char in color.lower())


def create_badge_blueprint(allowed_types):
    """Create the badge blueprint.

    :param allowed_types: A list of allowed types.
    :returns: A Flask blueprint.
    """
    from invenio_formatter.context_processors.badges import (
        generate_badge_png,
        generate_badge_svg,
    )

    blueprint = Blueprint(
        "invenio_formatter_badges",
        __name__,
        template_folder="templates",
    )

    @blueprint.route(
        "/badge/<any({0}):title>/<path:value>.<any(svg, png):ext>".format(
            ", ".join(allowed_types)
        )
    )
    def badge(title, value, ext="svg"):
        """Generate a badge response."""
        generator_kwargs = {}
        if ext == "svg":
            generator = generate_badge_svg
            mimetype = "image/svg+xml"
        elif ext == "png":
            generator = generate_badge_png
            mimetype = "image/png"

        color = request.args.get("color")
        if color and valid_css_color(color):
            generator_kwargs["color"] = "#" + request.args.get("color")

        badge_title_mapping = current_app.config["FORMATTER_BADGES_TITLE_MAPPING"].get(
            title, title
        )
        response = Response(
            generator(badge_title_mapping, value, **generator_kwargs), mimetype=mimetype
        )
        # Generate Etag from badge title and value.
        hashable_badge = "{0}.{1}".format(badge_title_mapping, value).encode("utf-8")
        response.set_etag(hashlib.sha1(hashable_badge).hexdigest())
        # Add headers to prevent caching.
        response.headers["Pragma"] = "no-cache"
        response.cache_control.no_cache = True
        response.cache_control.max_age = current_app.config[
            "FORMATTER_BADGES_MAX_CACHE_AGE"
        ]
        response.last_modified = dt.utcnow()
        extra = timedelta(seconds=current_app.config["FORMATTER_BADGES_MAX_CACHE_AGE"])
        response.expires = response.last_modified + extra
        return response.make_conditional(request)

    return blueprint



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2024 CERN.
# Copyright (C) 2024-2025 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Jinja utilities for Invenio applications.

Quick start
-----------
This section presents an example of the Invenio-Formatter features.
In this section we will create a simple page with a time formatter and a
`shields.io <http://shields.io>`_ badge.


First, let us create a new template page on which we will render some text,
and save it to some location (e.g.: ``/home/myuser/templates/index.html``)

.. code-block:: html

    Today is {{ mydate|to_arrow|format_arrow('YYYY-MM-DD') }}

    Your book: {{ badge_svg('isbn','9780399547331')|safe }}

Create a new Flask application and set some configuration for badge generation:

>>> from flask import Flask
>>> app = Flask('TestApp', template_folder='/home/myuser/templates')
>>> app.config.update(FORMATTER_BADGES_ALLOWED_TITLES=('isbn', 'ISBN', ))
>>> app.config.update(FORMATTER_BADGES_TITLE_MAPPING={'isbn': 'ISBN',})

.. note::

    By default the ``template_folder`` is the directory
    ``templates`` at the root of your flask application.

Load the Invenio-Formatter extension and run the application:

>>> from invenio_formatter import InvenioFormatter
>>> ext_fmt = InvenioFormatter(app)
>>> app.run() # doctest: +SKIP

You should now be able to access the index page with formatted date and badge
examples at `http://localhost:5000 <http://localhost:5000>`_.

Badges endpoints
~~~~~~~~~~~~~~~~
In addition of generating an image of a shield badge, a badge-rendering
endpoint is also available for easy embedding on other websites.

You can modify the template as follows:

.. code-block:: html

    {% set mydate = mydate|from_isodate -%}
    Today is {{ mydate.strftime('%Y-%m-%d') }}

    Your book (with badge URL: ):
        <img src="{{ url_for('invenio_formatter_badges.badge',
         title='isbn', value='9780399547331', ext='svg') }}"></img>

Your badge will be visible on the page as before, and also directly accessible
at `http://localhost:5000/badge/ISBN/9780399547331.svg
<http://localhost:5000/badge/ISBN/9780399547331.svg>`_.
"""

from .ext import InvenioFormatter

__version__ = "3.3.1"

__all__ = ("__version__", "InvenioFormatter")
